@@ -162,7 +162,7 @@ if __name__ == "__main__":
     ap.add_argument( '--logY', type=boolArg, help='whether to use log scale on Y axis', default=False)
     ap.add_argument( '--rampStepDuration', type=float, default=60, help='duration, in seconds, of ramp step' )
     ap.add_argument( '--SLODuration', type=float, default=240, help='SLO duration, in seconds' )
-    ap.add_argument( '--SLOResponseTimeMax', type=float, default=1.5, help='SLO RT threshold, in seconds' )
+    ap.add_argument( '--SLOResponseTimeMax', type=float, default=2.0, help='SLO RT threshold, in seconds' )
 
     args = ap.parse_args()
 
@@ -366,7 +366,17 @@ if __name__ == "__main__":
             if not fields:
                 logger.info( 'no fields in %s', inFilePath )
                 continue
-            frameNum = int(resultFileNames[i].lstrip("TestPlan_results_").rstrip(".csv"))
+            # frameNum = int(resultFileNames[i].lstrip("TestPlan_results_").rstrip(".csv"))
+            if 'TestPlan_results_' in resultFileNames[i]:
+                frameNum = int(resultFileNames[i].lstrip("TestPlan_results_").rstrip(".csv"))
+            elif resultFileNames[i].startswith('jmeterOut_'):
+                numPart = resultFileNames[i].split('/')[0].split('_')[1]
+                frameNum = int( numPart )
+            else:
+                # should not happen, but may help debugging
+                print( 'file name not recognized', resultFileNames[i] )
+                continue
+
             startTimes = []
             elapsedTimes = []
             labels = []
